#if defined(__linux__)
#include "forward.hpp"
#include <core/global/globals.hpp>
#include <dlfcn.h>
#include <fancy.hpp>
#include <stdexcept>

template <typename T> void loadFunc(void *so, T &function, const std::string &name)
{
    function = reinterpret_cast<T>(dlsym(so, name.c_str()));

    if (function == nullptr)
    {
        throw std::runtime_error("Failed to load function " + name);
    }
}

bool Soundux::PipeWireApi::setup()
{
    auto *libpulse = dlopen("libpipewire-0.3.so", RTLD_LAZY);
    if (!libpulse)
    {
        //* For flatpak
        libpulse = dlopen("/usr/lib/x86_64-linux-gnu/libpipewire-0.3.so.0", RTLD_LAZY);
    }

    if (libpulse)
    {
#define load(name) loadFunc(libpulse, name, #name)
        load(pw_context_connect);
        load(pw_context_new);
        load(pw_main_loop_new);
        load(pw_main_loop_get_loop);
        load(pw_proxy_add_listener);
        load(pw_properties_setf);
        load(pw_properties_set);
        load(pw_properties_new);
        load(pw_properties_free);
        load(pw_main_loop_destroy);
        load(pw_main_loop_quit);
        load(pw_context_destroy);
        load(pw_core_disconnect);
        load(pw_main_loop_run);
        load(pw_proxy_destroy);
        load(pw_init);
        return true;
    }

    Fancy::fancy.logTime().failure() << "Failed to load pipewire" << std::endl;
    Globals::gAudioBackend = std::make_shared<Objects::AudioBackend>();
    return false;
}

#endif